//
// C++ Interface: %{MODULE}
//
// Description: 
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef ADM_LEFTY
#define ADM_LEFTY
#include "adm_glyph.h"

uint8_t adm_estimate_glyphSize(admGlyph *gl,uint32_t *minx, uint32_t *maxx,uint32_t *miny,uint32_t *maxy,int *raw);
#endif 
