/***************************************************************************
                          ADM_vidChromaShift.h  -  description
                             -------------------

	Shift chroma to the left or to the right

    begin                : Wed Aug 28 2002
    copyright            : (C) 2002 by mean
    email                : fixounet@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CHROMASHIFT_PARAM_
#define CHROMASHIFT_PARAM_

typedef struct CHROMASHIFT_PARAM
{
      int32_t u;
      int32_t v;
}CHROMASHIFT_PARAM;

#endif
