//
// C++ Implementation: DIA_Msmooth
//
// Description: 
//
//
// Author: mean <fixounet@free.fr>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
//
#include "ADM_default.h"
#include "ADM_vidCNR2_param.h"

uint8_t DIA_cnr2(CNR2Param *param)
{
        return 1;

}

