/***************************************************************************
                          DIA_Busy.cpp  -  description
                             -------------------
    begin                : Thu Apr 21 2003
    copyright            : (C) 2003 by mean
    email                : fixounet@free.fr

	This function displays a busy box

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "DIA_busy.h"
#include "ADM_toolkitQt.h"

void DIA_StartBusy( void )
{

}

void DIA_StopBusy( void )
{
}


void DIA_StartBusyDialog( void )
{
	UI_purge();
}

void DIA_StopBusyDialog( void )
{
}

void DIA_runBusy( void )
{
	UI_purge();
}
