//AD  <- These first 4 characters need to be the first 4 characters to identify the ECMAScript file to Avidemux
var app = new Avidemux();
var file="/work/samples/2mn.avi";
var goodfcc="DIV3";
var fps;

	dialogFactoryHex();

/* End of test
*/
