
#define  MODULE_OGM 		1
#define  MODULE_CODEC 	        2
#define  MODULE_MPEG		4
#define  MODULE_EDITOR 	        8
#define  MODULE_GUI	 	0x10
#define  MODULE_SAVE_AVI 	0x20
#define  MODULE_FILTER		0x40
#define  MODULE_NUV		0x80
#define MODULE_ASF MODULE_NUV
#define  MODULE_MP4     	0x000100
#define  MODULE_CLOCKnTIMELEFT  0x000200
#define  MODULE_COMMANDLINE     0x000400
#define  MODULE_PREVIEW	 	0x000800
#define MODULE_AUDIO		0x001000
#define MODULE_ENCODER		0x002000
#define MODULE_AUDIO_FILTER     0x004000
#define  MODULE_DEBUG 		0x008000
#define MODULE_UI		0x010000
#define  MODULE_3GP 		0x020000
#define  MODULE_MPEG_DEMUX	0x040000
#define  MODULE_AUDIO_EDITOR    0x080000
#define MODULE_INPIC		0x100000
#define MODULE_ODML		0x400000
#define MODULE_OGM_AUDIO	0x800000
#define MODULE_REQUANT		0x1000000
#define MODULE_TCREQUANT	0x2000000
#define MODULE_UNPACKER		0x4000000
#define MODULE_ADEVICE		0x8000000
#define MODULE_SCRIPT		0x10000000
#define MODULE_OGM_WRITE	0x20000000
#define MODULE_XVID_RCTL	0x40000000
#define MODULE_LAVFORMAT	0x80000000    

