/***************************************************************************
                          DIA_contrast.cpp  -  description
                             -------------------
    begin                : Mon Sep 23 2002
    copyright            : (C) 2002 by mean
    email                : fixounet@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "ADM_default.h"
#include "ADM_videoFilter.h"
#include "ADM_vidContrast.h"


#include "DIA_flyDialog.h"
#include "DIA_flyContrast.h"
/********************************************************************/
uint8_t DIA_contrast(AVDMGenericVideoStream *in,CONTRAST_PARAM *param)
{
       return 1;
}
uint8_t flyContrast::upload(void)
{
    return 1;
}
uint8_t flyContrast::download (void)
{
   return 1;
}
//

