#ifndef _ADM_AVIDEMUXVIDEO_H
#define _ADM_AVIDEMUXVIDEO_H

class ADM_AvidemuxVideo
{
public:
	ADM_AvidemuxVideo(void) {}
	virtual ~ADM_AvidemuxVideo(void);

	// member variables
        bool m_bVideoProcess;
};

#endif
