/***************************************************************************
    copyright            : Mplayer team/Mean for ADM port
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef ADM_DELOGO_H
#define ADM_DELOGO_H
#include "ADM_default.h"
typedef struct MPDELOGO_PARAM
{
        uint32_t xoff;
        uint32_t yoff;
        uint32_t lw;
        uint32_t lh;
        uint32_t band;
        uint32_t show;
}MPDELOGO_PARAM;
#endif
