/*
 * DO NOT EDIT THIS FILE - it is generated by Glade.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "ADM_default.h"
#include "ADM_gladeSupport.h"

#include "../ADM_toolkit_gtk/jogshuttle.h"
#include "../ADM_toolkit_gtk/gtkmarkscale.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

GtkWidget*
create_mainWindow (void)
{
  GtkWidget *mainWindow;
  GdkPixbuf *mainWindow_icon_pixbuf;
  GtkWidget *vbox1;
  GtkWidget *menuBar;
  GtkWidget *menuitem1;
  GtkWidget *menuitem1_menu;
  GtkWidget *open_video1;
  GtkWidget *image8523;
  GtkWidget *append_video1;
  GtkWidget *image8524;
  GtkWidget *save_stuff;
  GtkWidget *image8525;
  GtkWidget *save_stuff_menu;
  GtkWidget *save_as_avi1;
  GtkWidget *separator1;
  GtkWidget *save_image1;
  GtkWidget *save_jpg_image1;
  GtkWidget *save_selection_as_jpegs1;
  GtkWidget *close1;
  GtkWidget *separator3;
  GtkWidget *save_project1;
  GtkWidget *image8526;
  GtkWidget *save_project_as1;
  GtkWidget *image8527;
  GtkWidget *run_script1;
  GtkWidget *image8528;
  GtkWidget *add_to_joblist1;
  GtkWidget *joblist1;
  GtkWidget *separator16;
  GtkWidget *connect_to_avsproxy1;
  GtkWidget *separator18;
  GtkWidget *video_informations1;
  GtkWidget *image8529;
  GtkWidget *separator4;
  GtkWidget *quit1;
  GtkWidget *image8530;
  GtkWidget *menuitem2;
  GtkWidget *menuitem2_menu;
  GtkWidget *reset_edits1;
  GtkWidget *image8531;
  GtkWidget *separator5;
  GtkWidget *cut1;
  GtkWidget *image8532;
  GtkWidget *copy1;
  GtkWidget *image8533;
  GtkWidget *paste1;
  GtkWidget *image8534;
  GtkWidget *delete1;
  GtkWidget *image8535;
  GtkWidget *separator6;
  GtkWidget *set_marker_a1;
  GtkWidget *set_marker_b1;
  GtkWidget *separator7;
  GtkWidget *preferences1;
  GtkWidget *image8536;
  GtkWidget *view1;
  GtkWidget *view1_menu;
  GtkWidget *toolbar1;
  GtkWidget *sidebar1;
  GtkWidget *separator8;
  GtkWidget *zoom_1_4;
  GtkWidget *zoom_1_2;
  GtkWidget *zoom_1_1;
  GtkWidget *zoom_2_1;
  GtkWidget *video_1;
  GtkWidget *video_1_menu;
  GtkWidget *decoder_options1;
  GtkWidget *set_postprocessing1;
  GtkWidget *separator9;
  GtkWidget *preview1;
  GtkWidget *display_output1;
  GtkWidget *separator10;
  GtkWidget *change_fps;
  GtkWidget *videoencoder;
  GtkWidget *filters1;
  GtkWidget *audio1;
  GtkWidget *audio1_menu;
  GtkWidget *main_audio;
  GtkWidget *second_audio_track1;
  GtkWidget *build_vbr_time_map1;
  GtkWidget *separator11;
  GtkWidget *save_audio1;
  GtkWidget *image8537;
  GtkWidget *audio_encoder1;
  GtkWidget *filters2;
  GtkWidget *tools1;
  GtkWidget *tools1_menu;
  GtkWidget *calculator1;
  GtkWidget *image8538;
  GtkWidget *separator12;
  GtkWidget *rebuild_frames;
  GtkWidget *check_frames;
  GtkWidget *bitrate_histogram1;
  GtkWidget *item1;
  GtkWidget *separator13;
  GtkWidget *v2v;
  GtkWidget *ocr_vobsub_2_srt;
  GtkWidget *ocr_dvb;
  GtkWidget *edit_glyph1;
  GtkWidget *separator17;
  GtkWidget *see_hex1;
  GtkWidget *help1;
  GtkWidget *go1;
  GtkWidget *go1_menu;
  GtkWidget *play_video1;
  GtkWidget *image8539;
  GtkWidget *previous_frame1;
  GtkWidget *image8540;
  GtkWidget *next_frame1;
  GtkWidget *image8541;
  GtkWidget *previous_intra_frame1;
  GtkWidget *image8542;
  GtkWidget *next_intra_frame1;
  GtkWidget *image8543;
  GtkWidget *search_previous_black_frame1;
  GtkWidget *search_next_black_frame1;
  GtkWidget *first_frame1;
  GtkWidget *image8544;
  GtkWidget *last_frame1;
  GtkWidget *image8545;
  GtkWidget *separator14;
  GtkWidget *go_to_marker_a1;
  GtkWidget *go_to_marker_b1;
  GtkWidget *separator15;
  GtkWidget *jum_to_frame1;
  GtkWidget *image8546;
  GtkWidget *jump_to_time1;
  GtkWidget *image8547;
  GtkWidget *custom1;
  GtkWidget *help2;
  GtkWidget *help2_menu;
  GtkWidget *plugins1;
  GtkWidget *show_builtin_support1;
  GtkWidget *about1;
  GtkWidget *image8548;
  GtkWidget *toolbar2;
  GtkIconSize tmp_toolbar_icon_size;
  GtkWidget *menutoolbuttonOpen;
  GtkWidget *toolitem11;
  GtkWidget *toolbuttonSave;
  GtkWidget *toolbuttonInfo;
  GtkWidget *separatortoolitem1;
  GtkWidget *tmp_image;
  GtkWidget *toolbuttonCalc;
  GtkWidget *separatortoolitem2;
  GtkWidget *toolitem12;
  GtkWidget *comboboxPreview;
  GtkWidget *toolitem13;
  GtkWidget *togglebutton1;
  GtkWidget *image8272;
  GtkWidget *toolitem14;
  GtkWidget *hscalVolume;
  GtkWidget *vbox10;
  GtkWidget *hbox14;
  GtkWidget *alignment19;
  GtkWidget *vbox9;
  GtkWidget *vbox11;
  GtkWidget *label18;
  GtkWidget *alignment12;
  GtkWidget *vbox7;
  GtkWidget *comboboxVideo;
  GtkWidget *buttonConfV;
  GtkWidget *buttonFilters;
  GtkWidget *vbox12;
  GtkWidget *label19;
  GtkWidget *alignment13;
  GtkWidget *vbox8;
  GtkWidget *comboboxAudio;
  GtkWidget *buttonConfA;
  GtkWidget *buttonAudioFilter;
  GtkWidget *alignment16;
  GtkWidget *hbox18;
  GtkWidget *label22;
  GtkWidget *hbox19;
  GtkWidget *CheckButtonTimeshift;
  GtkObject *spinbuttonTimeShift_adj;
  GtkWidget *spinbuttonTimeShift;
  GtkWidget *label23;
  GtkWidget *vbox13;
  GtkWidget *label20;
  GtkWidget *alignment14;
  GtkWidget *vbox15;
  GtkWidget *comboboxFormat;
  GtkWidget *buttonConfM;
  GtkWidget *guiDrawing;
  GtkWidget *table2;
  GtkWidget *hbox16;
  GtkWidget *label1;
  GtkWidget *boxCurFrame;
  GtkWidget *labelTotalFrame;
  GtkWidget *label4;
  GtkWidget *boxCurTime;
  GtkWidget *labelTotalTime;
  GtkWidget *labelFrameType;
  GtkWidget *sliderNavigate;
  GtkWidget *hbox10;
  GtkWidget *buttonPlay;
  GtkWidget *image591;
  GtkWidget *buttonStop;
  GtkWidget *image5056;
  GtkWidget *buttonPrevFrame;
  GtkWidget *image593;
  GtkWidget *buttonNextFrame;
  GtkWidget *image594;
  GtkWidget *buttonPrevKFrame;
  GtkWidget *image595;
  GtkWidget *buttonNextKFrame;
  GtkWidget *image596;
  GtkWidget *buttonMarkA;
  GtkWidget *image2306;
  GtkWidget *buttonMarkB;
  GtkWidget *image2307;
  GtkWidget *buttonPrevBlack;
  GtkWidget *image2308;
  GtkWidget *buttonNextBlack;
  GtkWidget *image2309;
  GtkWidget *buttonBegin;
  GtkWidget *image5057;
  GtkWidget *buttonEnd;
  GtkWidget *image2245;
  GtkWidget *vbox14;
  GtkWidget *label21;
  GtkWidget *alignment15;
  GtkWidget *table1;
  GtkWidget *buttonGotoB;
  GtkWidget *buttonGotoA;
  GtkWidget *labelMarkB;
  GtkWidget *labelMarkA;
  GtkWidget *jogg;
  extern GtkAccelGroup *accel_group;
  GtkTooltips *tooltips;

  tooltips = gtk_tooltips_new ();

  accel_group = gtk_accel_group_new ();

  mainWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (mainWindow), QT_TR_NOOP("Avidemux"));
  mainWindow_icon_pixbuf = create_pixbuf ("avidemux_icon_small.png");
  if (mainWindow_icon_pixbuf)
    {
      gtk_window_set_icon (GTK_WINDOW (mainWindow), mainWindow_icon_pixbuf);
      gdk_pixbuf_unref (mainWindow_icon_pixbuf);
    }

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (mainWindow), vbox1);

  menuBar = gtk_menu_bar_new ();
  gtk_widget_show (menuBar);
  gtk_box_pack_start (GTK_BOX (vbox1), menuBar, FALSE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS (menuBar, GTK_CAN_FOCUS);

  menuitem1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_File"));
  gtk_widget_show (menuitem1);
  gtk_container_add (GTK_CONTAINER (menuBar), menuitem1);

  menuitem1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem1), menuitem1_menu);

  open_video1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_Open..."));
  gtk_widget_show (open_video1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), open_video1);
  gtk_widget_add_accelerator (open_video1, "activate", accel_group,
                              GDK_O, (GdkModifierType)( GDK_CONTROL_MASK),
                              GTK_ACCEL_VISIBLE);

  image8523 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8523);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (open_video1), image8523);

  append_video1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_Append..."));
  gtk_widget_show (append_video1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), append_video1);
  gtk_widget_add_accelerator (append_video1, "activate", accel_group,
                              GDK_A, (GdkModifierType)( GDK_CONTROL_MASK | GDK_MOD1_MASK),
                              GTK_ACCEL_VISIBLE);

  image8524 = gtk_image_new_from_stock ("gtk-add", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8524);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (append_video1), image8524);

  save_stuff = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_Save"));
  gtk_widget_show (save_stuff);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), save_stuff);

  image8525 = gtk_image_new_from_stock ("gtk-save", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8525);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (save_stuff), image8525);

  save_stuff_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (save_stuff), save_stuff_menu);

  save_as_avi1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Save _Video..."));
  gtk_widget_show (save_as_avi1);
  gtk_container_add (GTK_CONTAINER (save_stuff_menu), save_as_avi1);
  gtk_widget_add_accelerator (save_as_avi1, "activate", accel_group,
                              GDK_S, (GdkModifierType)( GDK_CONTROL_MASK),
                              GTK_ACCEL_VISIBLE);

  separator1 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator1);
  gtk_container_add (GTK_CONTAINER (save_stuff_menu), separator1);
  gtk_widget_set_sensitive (separator1, FALSE);

  save_image1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Save _BMP Image..."));
  gtk_widget_show (save_image1);
  gtk_container_add (GTK_CONTAINER (save_stuff_menu), save_image1);
  gtk_widget_add_accelerator (save_image1, "activate", accel_group,
                              GDK_M, (GdkModifierType)( GDK_CONTROL_MASK),
                              GTK_ACCEL_VISIBLE);

  save_jpg_image1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Save _JPEG Image..."));
  gtk_widget_show (save_jpg_image1);
  gtk_container_add (GTK_CONTAINER (save_stuff_menu), save_jpg_image1);
  gtk_widget_add_accelerator (save_jpg_image1, "activate", accel_group,
                              GDK_E, (GdkModifierType)( GDK_CONTROL_MASK),
                              GTK_ACCEL_VISIBLE);

  save_selection_as_jpegs1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Save _Selection as JPEG Images..."));
  gtk_widget_show (save_selection_as_jpegs1);
  gtk_container_add (GTK_CONTAINER (save_stuff_menu), save_selection_as_jpegs1);

  close1 = gtk_image_menu_item_new_from_stock ("gtk-close", accel_group);
  gtk_widget_show (close1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), close1);

  separator3 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator3);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), separator3);
  gtk_widget_set_sensitive (separator3, FALSE);

  save_project1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("Save _Project"));
  gtk_widget_show (save_project1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), save_project1);

  image8526 = gtk_image_new_from_stock ("gtk-save", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8526);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (save_project1), image8526);

  save_project_as1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("Save P_roject As..."));
  gtk_widget_show (save_project_as1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), save_project_as1);

  image8527 = gtk_image_new_from_stock ("gtk-save-as", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8527);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (save_project_as1), image8527);

  run_script1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_Load/Run Project..."));
  gtk_widget_show (run_script1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), run_script1);

  image8528 = gtk_image_new_from_stock ("gtk-execute", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8528);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (run_script1), image8528);

  add_to_joblist1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("A_dd to Joblist..."));
  gtk_widget_show (add_to_joblist1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), add_to_joblist1);

  joblist1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Show _Joblist"));
  gtk_widget_show (joblist1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), joblist1);
  gtk_widget_add_accelerator (joblist1, "activate", accel_group,
                              GDK_J, (GdkModifierType)( GDK_CONTROL_MASK),
                              GTK_ACCEL_VISIBLE);

  separator16 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator16);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), separator16);
  gtk_widget_set_sensitive (separator16, FALSE);

  connect_to_avsproxy1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Co_nnect to avsproxy"));
  gtk_widget_show (connect_to_avsproxy1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), connect_to_avsproxy1);

  separator18 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator18);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), separator18);
  gtk_widget_set_sensitive (separator18, FALSE);

  video_informations1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("Proper_ties"));
  gtk_widget_show (video_informations1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), video_informations1);
  gtk_widget_add_accelerator (video_informations1, "activate", accel_group,
                              GDK_Return, (GdkModifierType)( GDK_MOD1_MASK),
                              GTK_ACCEL_VISIBLE);

  image8529 = gtk_image_new_from_stock ("gtk-properties", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8529);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (video_informations1), image8529);

  separator4 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator4);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), separator4);
  gtk_widget_set_sensitive (separator4, FALSE);

  quit1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_Quit"));
  gtk_widget_show (quit1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), quit1);
  gtk_widget_add_accelerator (quit1, "activate", accel_group,
                              GDK_Q, (GdkModifierType)( GDK_CONTROL_MASK),
                              GTK_ACCEL_VISIBLE);

  image8530 = gtk_image_new_from_stock ("gtk-quit", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8530);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (quit1), image8530);

  menuitem2 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Edit"));
  gtk_widget_show (menuitem2);
  gtk_container_add (GTK_CONTAINER (menuBar), menuitem2);

  menuitem2_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem2), menuitem2_menu);

  reset_edits1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_Reset Edits"));
  gtk_widget_show (reset_edits1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), reset_edits1);

  image8531 = gtk_image_new_from_stock ("gtk-undo", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8531);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (reset_edits1), image8531);

  separator5 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator5);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), separator5);
  gtk_widget_set_sensitive (separator5, FALSE);

  cut1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("Cu_t"));
  gtk_widget_show (cut1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), cut1);
  gtk_widget_add_accelerator (cut1, "activate", accel_group,
                              GDK_X, (GdkModifierType)( GDK_CONTROL_MASK),
                              GTK_ACCEL_VISIBLE);

  image8532 = gtk_image_new_from_stock ("gtk-cut", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8532);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (cut1), image8532);

  copy1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_Copy"));
  gtk_widget_show (copy1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), copy1);
  gtk_widget_add_accelerator (copy1, "activate", accel_group,
                              GDK_C, (GdkModifierType)( GDK_CONTROL_MASK),
                              GTK_ACCEL_VISIBLE);

  image8533 = gtk_image_new_from_stock ("gtk-copy", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8533);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (copy1), image8533);

  paste1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_Paste"));
  gtk_widget_show (paste1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), paste1);
  gtk_widget_add_accelerator (paste1, "activate", accel_group,
                              GDK_V, (GdkModifierType)( GDK_CONTROL_MASK),
                              GTK_ACCEL_VISIBLE);

  image8534 = gtk_image_new_from_stock ("gtk-paste", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8534);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (paste1), image8534);

  delete1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_Delete"));
  gtk_widget_show (delete1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), delete1);
  gtk_widget_add_accelerator (delete1, "activate", accel_group,
                              GDK_Delete, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  image8535 = gtk_image_new_from_stock ("gtk-remove", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8535);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (delete1), image8535);

  separator6 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator6);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), separator6);
  gtk_widget_set_sensitive (separator6, FALSE);

  set_marker_a1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Set Marker _A"));
  gtk_widget_show (set_marker_a1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), set_marker_a1);
  gtk_widget_add_accelerator (set_marker_a1, "activate", accel_group,
                              GDK_bracketleft, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  set_marker_b1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Set Marker _B"));
  gtk_widget_show (set_marker_b1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), set_marker_b1);
  gtk_widget_add_accelerator (set_marker_b1, "activate", accel_group,
                              GDK_bracketright, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  separator7 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator7);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), separator7);
  gtk_widget_set_sensitive (separator7, FALSE);

  preferences1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("Pre_ferences"));
  gtk_widget_show (preferences1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), preferences1);

  image8536 = gtk_image_new_from_stock ("gtk-preferences", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8536);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (preferences1), image8536);

  view1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_View"));
  gtk_widget_show (view1);
  gtk_container_add (GTK_CONTAINER (menuBar), view1);

  view1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (view1), view1_menu);

  toolbar1 = gtk_check_menu_item_new_with_mnemonic (QT_TR_NOOP("_Main Toolbar"));
  gtk_widget_show (toolbar1);
  gtk_container_add (GTK_CONTAINER (view1_menu), toolbar1);
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (toolbar1), TRUE);

  sidebar1 = gtk_check_menu_item_new_with_mnemonic (QT_TR_NOOP("_A/V Sidebar"));
  gtk_widget_show (sidebar1);
  gtk_container_add (GTK_CONTAINER (view1_menu), sidebar1);
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (sidebar1), TRUE);

  separator8 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator8);
  gtk_container_add (GTK_CONTAINER (view1_menu), separator8);
  gtk_widget_set_sensitive (separator8, FALSE);

  zoom_1_4 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Zoom 1:4"));
  gtk_widget_show (zoom_1_4);
  gtk_container_add (GTK_CONTAINER (view1_menu), zoom_1_4);
  gtk_widget_add_accelerator (zoom_1_4, "activate", accel_group,
                              GDK_4, (GdkModifierType)( GDK_CONTROL_MASK | GDK_SHIFT_MASK),
                              GTK_ACCEL_VISIBLE);

  zoom_1_2 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Z_oom 1:2"));
  gtk_widget_show (zoom_1_2);
  gtk_container_add (GTK_CONTAINER (view1_menu), zoom_1_2);
  gtk_widget_add_accelerator (zoom_1_2, "activate", accel_group,
                              GDK_2, (GdkModifierType)( GDK_CONTROL_MASK | GDK_SHIFT_MASK),
                              GTK_ACCEL_VISIBLE);

  zoom_1_1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Zoom _1:1"));
  gtk_widget_show (zoom_1_1);
  gtk_container_add (GTK_CONTAINER (view1_menu), zoom_1_1);
  gtk_widget_add_accelerator (zoom_1_1, "activate", accel_group,
                              GDK_1, (GdkModifierType)( GDK_CONTROL_MASK | GDK_MOD1_MASK),
                              GTK_ACCEL_VISIBLE);

  zoom_2_1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Zoom _2:1"));
  gtk_widget_show (zoom_2_1);
  gtk_container_add (GTK_CONTAINER (view1_menu), zoom_2_1);
  gtk_widget_add_accelerator (zoom_2_1, "activate", accel_group,
                              GDK_2, (GdkModifierType)( GDK_CONTROL_MASK | GDK_MOD1_MASK),
                              GTK_ACCEL_VISIBLE);

  video_1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Vi_deo"));
  gtk_widget_show (video_1);
  gtk_container_add (GTK_CONTAINER (menuBar), video_1);

  video_1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (video_1), video_1_menu);

  decoder_options1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Decoder Options"));
  gtk_widget_show (decoder_options1);
  gtk_container_add (GTK_CONTAINER (video_1_menu), decoder_options1);
  gtk_widget_add_accelerator (decoder_options1, "activate", accel_group,
                              GDK_F3, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  set_postprocessing1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Postprocessing"));
  gtk_widget_show (set_postprocessing1);
  gtk_container_add (GTK_CONTAINER (video_1_menu), set_postprocessing1);
  gtk_tooltips_set_tip (tooltips, set_postprocessing1, QT_TR_NOOP("Select postprocessing level"), NULL);
  gtk_widget_add_accelerator (set_postprocessing1, "activate", accel_group,
                              GDK_F4, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  separator9 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator9);
  gtk_container_add (GTK_CONTAINER (video_1_menu), separator9);
  gtk_widget_set_sensitive (separator9, FALSE);

  preview1 = gtk_check_menu_item_new_with_mnemonic (QT_TR_NOOP("P_review"));
  gtk_widget_show (preview1);
  gtk_container_add (GTK_CONTAINER (video_1_menu), preview1);
  gtk_widget_add_accelerator (preview1, "activate", accel_group,
                              GDK_F5, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  display_output1 = gtk_check_menu_item_new_with_mnemonic (QT_TR_NOOP("Display _Output"));
  gtk_widget_show (display_output1);
  gtk_container_add (GTK_CONTAINER (video_1_menu), display_output1);
  gtk_widget_add_accelerator (display_output1, "activate", accel_group,
                              GDK_F6, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  separator10 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator10);
  gtk_container_add (GTK_CONTAINER (video_1_menu), separator10);
  gtk_widget_set_sensitive (separator10, FALSE);

  change_fps = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Frame Rate"));
  gtk_widget_show (change_fps);
  gtk_container_add (GTK_CONTAINER (video_1_menu), change_fps);

  videoencoder = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Encoder"));
  gtk_widget_show (videoencoder);
  gtk_container_add (GTK_CONTAINER (video_1_menu), videoencoder);
  gtk_widget_add_accelerator (videoencoder, "activate", accel_group,
                              GDK_V, (GdkModifierType)( GDK_CONTROL_MASK | GDK_MOD1_MASK),
                              GTK_ACCEL_VISIBLE);

  filters1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Fil_ters"));
  gtk_widget_show (filters1);
  gtk_container_add (GTK_CONTAINER (video_1_menu), filters1);
  gtk_widget_add_accelerator (filters1, "activate", accel_group,
                              GDK_F, (GdkModifierType)( GDK_CONTROL_MASK | GDK_MOD1_MASK),
                              GTK_ACCEL_VISIBLE);

  audio1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Audio"));
  gtk_widget_show (audio1);
  gtk_container_add (GTK_CONTAINER (menuBar), audio1);

  audio1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (audio1), audio1_menu);

  main_audio = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Main Track"));
  gtk_widget_show (main_audio);
  gtk_container_add (GTK_CONTAINER (audio1_menu), main_audio);

  second_audio_track1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Second Track"));
  gtk_widget_show (second_audio_track1);
  gtk_container_add (GTK_CONTAINER (audio1_menu), second_audio_track1);

  build_vbr_time_map1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Build VBR Time Map"));
  gtk_widget_show (build_vbr_time_map1);
  gtk_container_add (GTK_CONTAINER (audio1_menu), build_vbr_time_map1);

  separator11 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator11);
  gtk_container_add (GTK_CONTAINER (audio1_menu), separator11);
  gtk_widget_set_sensitive (separator11, FALSE);

  save_audio1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("S_ave..."));
  gtk_widget_show (save_audio1);
  gtk_container_add (GTK_CONTAINER (audio1_menu), save_audio1);
  gtk_widget_add_accelerator (save_audio1, "activate", accel_group,
                              GDK_S, (GdkModifierType)( GDK_CONTROL_MASK | GDK_MOD1_MASK),
                              GTK_ACCEL_VISIBLE);

  image8537 = gtk_image_new_from_stock ("gtk-save-as", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8537);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (save_audio1), image8537);

  audio_encoder1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Encoder"));
  gtk_widget_show (audio_encoder1);
  gtk_container_add (GTK_CONTAINER (audio1_menu), audio_encoder1);
  gtk_widget_add_accelerator (audio_encoder1, "activate", accel_group,
                              GDK_A, (GdkModifierType)( GDK_CONTROL_MASK | GDK_MOD1_MASK),
                              GTK_ACCEL_VISIBLE);

  filters2 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Filters"));
  gtk_widget_show (filters2);
  gtk_container_add (GTK_CONTAINER (audio1_menu), filters2);

  tools1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Tools"));
  gtk_widget_show (tools1);
  gtk_container_add (GTK_CONTAINER (menuBar), tools1);

  tools1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (tools1), tools1_menu);

  calculator1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_Calculator"));
  gtk_widget_show (calculator1);
  gtk_container_add (GTK_CONTAINER (tools1_menu), calculator1);
  gtk_widget_add_accelerator (calculator1, "activate", accel_group,
                              GDK_F7, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  image8538 = create_pixmap (mainWindow, "gnome-calculator_small.xpm");
  gtk_widget_show (image8538);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (calculator1), image8538);

  separator12 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator12);
  gtk_container_add (GTK_CONTAINER (tools1_menu), separator12);
  gtk_widget_set_sensitive (separator12, FALSE);

  rebuild_frames = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Rebuild Frames (I & B)"));
  gtk_widget_show (rebuild_frames);
  gtk_container_add (GTK_CONTAINER (tools1_menu), rebuild_frames);

  check_frames = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("C_heck Frames (slow)"));
  gtk_widget_show (check_frames);
  gtk_container_add (GTK_CONTAINER (tools1_menu), check_frames);

  bitrate_histogram1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Bitrate Histogram"));
  gtk_widget_show (bitrate_histogram1);
  gtk_container_add (GTK_CONTAINER (tools1_menu), bitrate_histogram1);

  item1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Scan for Black Frames..."));
  gtk_widget_show (item1);
  gtk_container_add (GTK_CONTAINER (tools1_menu), item1);

  separator13 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator13);
  gtk_container_add (GTK_CONTAINER (tools1_menu), separator13);
  gtk_widget_set_sensitive (separator13, FALSE);

  v2v = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_VOB -> VobSub"));
  gtk_widget_show (v2v);
  gtk_container_add (GTK_CONTAINER (tools1_menu), v2v);

  ocr_vobsub_2_srt = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_OCR (VobSub -> srt)"));
  gtk_widget_show (ocr_vobsub_2_srt);
  gtk_container_add (GTK_CONTAINER (tools1_menu), ocr_vobsub_2_srt);

  ocr_dvb = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("OCR (TS->srt)"));
  gtk_widget_show (ocr_dvb);
  gtk_container_add (GTK_CONTAINER (tools1_menu), ocr_dvb);

  edit_glyph1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Edit Glyph"));
  gtk_widget_show (edit_glyph1);
  gtk_container_add (GTK_CONTAINER (tools1_menu), edit_glyph1);

  separator17 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator17);
  gtk_container_add (GTK_CONTAINER (tools1_menu), separator17);
  gtk_widget_set_sensitive (separator17, FALSE);

  see_hex1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Frame Hex Dump"));
  gtk_widget_show (see_hex1);
  gtk_container_add (GTK_CONTAINER (tools1_menu), see_hex1);

  help1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("A_uto"));
  gtk_widget_show (help1);
  gtk_container_add (GTK_CONTAINER (menuBar), help1);

  go1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Go"));
  gtk_widget_show (go1);
  gtk_container_add (GTK_CONTAINER (menuBar), go1);

  go1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (go1), go1_menu);

  play_video1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_Play/Stop"));
  gtk_widget_show (play_video1);
  gtk_container_add (GTK_CONTAINER (go1_menu), play_video1);
  gtk_widget_add_accelerator (play_video1, "activate", accel_group,
                              GDK_P, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  image8539 = gtk_image_new_from_stock ("gtk-media-play", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8539);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (play_video1), image8539);

  previous_frame1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("P_revious Frame"));
  gtk_widget_show (previous_frame1);
  gtk_container_add (GTK_CONTAINER (go1_menu), previous_frame1);
  gtk_widget_add_accelerator (previous_frame1, "activate", accel_group,
                              GDK_KP_4, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  image8540 = gtk_image_new_from_stock ("gtk-media-previous", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8540);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (previous_frame1), image8540);

  next_frame1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_Next Frame"));
  gtk_widget_show (next_frame1);
  gtk_container_add (GTK_CONTAINER (go1_menu), next_frame1);
  gtk_widget_add_accelerator (next_frame1, "activate", accel_group,
                              GDK_KP_6, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  image8541 = gtk_image_new_from_stock ("gtk-media-next", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8541);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (next_frame1), image8541);

  previous_intra_frame1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("Pr_evious Intra Frame"));
  gtk_widget_show (previous_intra_frame1);
  gtk_container_add (GTK_CONTAINER (go1_menu), previous_intra_frame1);
  gtk_widget_add_accelerator (previous_intra_frame1, "activate", accel_group,
                              GDK_KP_2, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  image8542 = gtk_image_new_from_stock ("gtk-media-rewind", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8542);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (previous_intra_frame1), image8542);

  next_intra_frame1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("Next _Intra Frame"));
  gtk_widget_show (next_intra_frame1);
  gtk_container_add (GTK_CONTAINER (go1_menu), next_intra_frame1);
  gtk_widget_add_accelerator (next_intra_frame1, "activate", accel_group,
                              GDK_KP_8, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  image8543 = gtk_image_new_from_stock ("gtk-media-forward", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8543);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (next_intra_frame1), image8543);

  search_previous_black_frame1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Previou_s Black Frame"));
  gtk_widget_show (search_previous_black_frame1);
  gtk_container_add (GTK_CONTAINER (go1_menu), search_previous_black_frame1);

  search_next_black_frame1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Ne_xt Black Frame"));
  gtk_widget_show (search_next_black_frame1);
  gtk_container_add (GTK_CONTAINER (go1_menu), search_next_black_frame1);

  first_frame1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_First Frame"));
  gtk_widget_show (first_frame1);
  gtk_container_add (GTK_CONTAINER (go1_menu), first_frame1);
  gtk_widget_add_accelerator (first_frame1, "activate", accel_group,
                              GDK_Home, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  image8544 = gtk_image_new_from_stock ("gtk-goto-first", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8544);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (first_frame1), image8544);

  last_frame1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_Last Frame"));
  gtk_widget_show (last_frame1);
  gtk_container_add (GTK_CONTAINER (go1_menu), last_frame1);
  gtk_widget_add_accelerator (last_frame1, "activate", accel_group,
                              GDK_End, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  image8545 = gtk_image_new_from_stock ("gtk-goto-last", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8545);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (last_frame1), image8545);

  separator14 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator14);
  gtk_container_add (GTK_CONTAINER (go1_menu), separator14);
  gtk_widget_set_sensitive (separator14, FALSE);

  go_to_marker_a1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Go to Marker _A"));
  gtk_widget_show (go_to_marker_a1);
  gtk_container_add (GTK_CONTAINER (go1_menu), go_to_marker_a1);
  gtk_widget_add_accelerator (go_to_marker_a1, "activate", accel_group,
                              GDK_bracketleft, (GdkModifierType)( GDK_CONTROL_MASK),
                              GTK_ACCEL_VISIBLE);

  go_to_marker_b1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("Go to Marker _B"));
  gtk_widget_show (go_to_marker_b1);
  gtk_container_add (GTK_CONTAINER (go1_menu), go_to_marker_b1);
  gtk_widget_add_accelerator (go_to_marker_b1, "activate", accel_group,
                              GDK_bracketright, (GdkModifierType)( GDK_CONTROL_MASK),
                              GTK_ACCEL_VISIBLE);

  separator15 = gtk_separator_menu_item_new ();
  gtk_widget_show (separator15);
  gtk_container_add (GTK_CONTAINER (go1_menu), separator15);
  gtk_widget_set_sensitive (separator15, FALSE);

  jum_to_frame1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("Go to Fra_me..."));
  gtk_widget_show (jum_to_frame1);
  gtk_container_add (GTK_CONTAINER (go1_menu), jum_to_frame1);
  gtk_widget_add_accelerator (jum_to_frame1, "activate", accel_group,
                              GDK_F, (GdkModifierType)( GDK_CONTROL_MASK),
                              GTK_ACCEL_VISIBLE);

  image8546 = gtk_image_new_from_stock ("gtk-jump-to", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8546);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (jum_to_frame1), image8546);

  jump_to_time1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("Go to _Time..."));
  gtk_widget_show (jump_to_time1);
  gtk_container_add (GTK_CONTAINER (go1_menu), jump_to_time1);
  gtk_widget_add_accelerator (jump_to_time1, "activate", accel_group,
                              GDK_T, (GdkModifierType)( GDK_CONTROL_MASK),
                              GTK_ACCEL_VISIBLE);

  image8547 = gtk_image_new_from_stock ("gtk-jump-to", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8547);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (jump_to_time1), image8547);

  custom1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Custom"));
  gtk_widget_show (custom1);
  gtk_container_add (GTK_CONTAINER (menuBar), custom1);

  help2 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Help"));
  gtk_widget_show (help2);
  gtk_container_add (GTK_CONTAINER (menuBar), help2);

  help2_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (help2), help2_menu);

  plugins1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Plugins"));
  gtk_widget_show (plugins1);
  gtk_container_add (GTK_CONTAINER (help2_menu), plugins1);

  show_builtin_support1 = gtk_menu_item_new_with_mnemonic (QT_TR_NOOP("_Built-in Support"));
  gtk_widget_show (show_builtin_support1);
  gtk_container_add (GTK_CONTAINER (help2_menu), show_builtin_support1);

  about1 = gtk_image_menu_item_new_with_mnemonic (QT_TR_NOOP("_About"));
  gtk_widget_show (about1);
  gtk_container_add (GTK_CONTAINER (help2_menu), about1);

  image8548 = gtk_image_new_from_stock ("gtk-about", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image8548);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (about1), image8548);

  toolbar2 = gtk_toolbar_new ();
  gtk_widget_show (toolbar2);
  gtk_box_pack_start (GTK_BOX (vbox1), toolbar2, FALSE, FALSE, 0);
  gtk_toolbar_set_style (GTK_TOOLBAR (toolbar2), GTK_TOOLBAR_BOTH_HORIZ);
  tmp_toolbar_icon_size = gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar2));

  menutoolbuttonOpen = (GtkWidget*) gtk_menu_tool_button_new_from_stock ("gtk-open");
  gtk_widget_show (menutoolbuttonOpen);
  gtk_tool_item_set_homogeneous (GTK_TOOL_ITEM (menutoolbuttonOpen), FALSE);
  gtk_container_add (GTK_CONTAINER (toolbar2), menutoolbuttonOpen);
  gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (menutoolbuttonOpen), tooltips, QT_TR_NOOP("Open a file"), NULL);
  gtk_tool_item_set_is_important (GTK_TOOL_ITEM (menutoolbuttonOpen), TRUE);

  toolitem11 = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (toolitem11);
  gtk_container_add (GTK_CONTAINER (toolbar2), toolitem11);

  toolbuttonSave = (GtkWidget*) gtk_tool_button_new_from_stock ("gtk-save");
  gtk_widget_show (toolbuttonSave);
  gtk_container_add (GTK_CONTAINER (toolbar2), toolbuttonSave);
  gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbuttonSave), tooltips, QT_TR_NOOP("Save the file"), NULL);
  gtk_tool_item_set_is_important (GTK_TOOL_ITEM (toolbuttonSave), TRUE);

  toolbuttonInfo = (GtkWidget*) gtk_tool_button_new_from_stock ("gtk-properties");
  gtk_widget_show (toolbuttonInfo);
  gtk_container_add (GTK_CONTAINER (toolbar2), toolbuttonInfo);
  gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbuttonInfo), tooltips, QT_TR_NOOP("Audio/video file information"), NULL);

  separatortoolitem1 = (GtkWidget*) gtk_separator_tool_item_new ();
  gtk_widget_show (separatortoolitem1);
  gtk_container_add (GTK_CONTAINER (toolbar2), separatortoolitem1);

  tmp_image = create_pixmap (mainWindow, "gnome-calculator.png");
  gtk_widget_show (tmp_image);
  toolbuttonCalc = (GtkWidget*) gtk_tool_button_new (tmp_image, QT_TR_NOOP("Calculator"));
  gtk_widget_show (toolbuttonCalc);
  gtk_container_add (GTK_CONTAINER (toolbar2), toolbuttonCalc);
  gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbuttonCalc), tooltips, QT_TR_NOOP("Bitrate/size calculator"), NULL);
  gtk_tool_item_set_is_important (GTK_TOOL_ITEM (toolbuttonCalc), TRUE);

  separatortoolitem2 = (GtkWidget*) gtk_separator_tool_item_new ();
  gtk_widget_show (separatortoolitem2);
  gtk_container_add (GTK_CONTAINER (toolbar2), separatortoolitem2);

  toolitem12 = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (toolitem12);
  gtk_container_add (GTK_CONTAINER (toolbar2), toolitem12);

  comboboxPreview = gtk_combo_box_new_text ();
  gtk_widget_show (comboboxPreview);
  gtk_container_add (GTK_CONTAINER (toolitem12), comboboxPreview);

  toolitem13 = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (toolitem13);
  gtk_container_add (GTK_CONTAINER (toolbar2), toolitem13);

  togglebutton1 = gtk_toggle_button_new ();
  gtk_widget_show (togglebutton1);
  gtk_container_add (GTK_CONTAINER (toolitem13), togglebutton1);
  gtk_button_set_relief (GTK_BUTTON (togglebutton1), GTK_RELIEF_NONE);

  image8272 = create_pixmap (mainWindow, "audio-volume-medium.png");
  gtk_widget_show (image8272);
  gtk_container_add (GTK_CONTAINER (togglebutton1), image8272);
  gtk_widget_set_size_request (image8272, 17, 16);

  toolitem14 = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (toolitem14);
  gtk_container_add (GTK_CONTAINER (toolbar2), toolitem14);

  hscalVolume = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (99, 0, 100, 1, 1, 0)));
  gtk_widget_show (hscalVolume);
  gtk_container_add (GTK_CONTAINER (toolitem14), hscalVolume);
  gtk_widget_set_size_request (hscalVolume, 100, -1);
  gtk_scale_set_draw_value (GTK_SCALE (hscalVolume), FALSE);
  gtk_scale_set_digits (GTK_SCALE (hscalVolume), 0);

  vbox10 = gtk_vbox_new (FALSE, 3);
  gtk_widget_show (vbox10);
  gtk_box_pack_start (GTK_BOX (vbox1), vbox10, TRUE, TRUE, 0);

  hbox14 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox14);
  gtk_box_pack_start (GTK_BOX (vbox10), hbox14, TRUE, TRUE, 0);

  alignment19 = gtk_alignment_new (0.5, 0.5, 1, 1);
  gtk_widget_show (alignment19);
  gtk_box_pack_start (GTK_BOX (hbox14), alignment19, FALSE, FALSE, 0);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment19), 3, 0, 3, 0);

  vbox9 = gtk_vbox_new (FALSE, 9);
  gtk_widget_show (vbox9);
  gtk_container_add (GTK_CONTAINER (alignment19), vbox9);

  vbox11 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox11);
  gtk_box_pack_start (GTK_BOX (vbox9), vbox11, FALSE, FALSE, 0);

  label18 = gtk_label_new (QT_TR_NOOP("<b>Video</b>"));
  gtk_widget_show (label18);
  gtk_box_pack_start (GTK_BOX (vbox11), label18, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label18), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label18), 0, 0.5);

  alignment12 = gtk_alignment_new (0.5, 0.5, 1, 1);
  gtk_widget_show (alignment12);
  gtk_box_pack_start (GTK_BOX (vbox11), alignment12, TRUE, TRUE, 0);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment12), 3, 0, 12, 0);

  vbox7 = gtk_vbox_new (FALSE, 3);
  gtk_widget_show (vbox7);
  gtk_container_add (GTK_CONTAINER (alignment12), vbox7);

  comboboxVideo = gtk_combo_box_new_text ();
  gtk_widget_show (comboboxVideo);
  gtk_box_pack_start (GTK_BOX (vbox7), comboboxVideo, FALSE, FALSE, 0);
  gtk_combo_box_append_text (GTK_COMBO_BOX (comboboxVideo), QT_TR_NOOP("Copy"));

  buttonConfV = gtk_button_new_with_mnemonic (QT_TR_NOOP("Configure"));
  gtk_widget_show (buttonConfV);
  gtk_box_pack_start (GTK_BOX (vbox7), buttonConfV, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonConfV, QT_TR_NOOP("Configure video encoder"), NULL);
  gtk_widget_add_accelerator (buttonConfV, "clicked", accel_group,
                              GDK_F3, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  buttonFilters = gtk_button_new_with_mnemonic (QT_TR_NOOP("Filters"));
  gtk_widget_show (buttonFilters);
  gtk_box_pack_start (GTK_BOX (vbox7), buttonFilters, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonFilters, QT_TR_NOOP("Video filters"), NULL);
  gtk_widget_add_accelerator (buttonFilters, "clicked", accel_group,
                              GDK_F1, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  vbox12 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox12);
  gtk_box_pack_start (GTK_BOX (vbox9), vbox12, FALSE, FALSE, 0);

  label19 = gtk_label_new (QT_TR_NOOP("<b>Audio</b>"));
  gtk_widget_show (label19);
  gtk_box_pack_start (GTK_BOX (vbox12), label19, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label19), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label19), 0, 0.5);

  alignment13 = gtk_alignment_new (0.5, 0.5, 1, 1);
  gtk_widget_show (alignment13);
  gtk_box_pack_start (GTK_BOX (vbox12), alignment13, TRUE, TRUE, 0);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment13), 3, 0, 12, 0);

  vbox8 = gtk_vbox_new (FALSE, 3);
  gtk_widget_show (vbox8);
  gtk_container_add (GTK_CONTAINER (alignment13), vbox8);

  comboboxAudio = gtk_combo_box_new_text ();
  gtk_widget_show (comboboxAudio);
  gtk_box_pack_start (GTK_BOX (vbox8), comboboxAudio, FALSE, FALSE, 0);
  gtk_combo_box_append_text (GTK_COMBO_BOX (comboboxAudio), QT_TR_NOOP("Copy"));

  buttonConfA = gtk_button_new_with_mnemonic (QT_TR_NOOP("Configure"));
  gtk_widget_show (buttonConfA);
  gtk_box_pack_start (GTK_BOX (vbox8), buttonConfA, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonConfA, QT_TR_NOOP("Configure audio encoder"), NULL);
  gtk_widget_add_accelerator (buttonConfA, "clicked", accel_group,
                              GDK_F4, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  buttonAudioFilter = gtk_button_new ();
  gtk_widget_show (buttonAudioFilter);
  gtk_box_pack_start (GTK_BOX (vbox8), buttonAudioFilter, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonAudioFilter, QT_TR_NOOP("Audio filters"), NULL);
  gtk_widget_add_accelerator (buttonAudioFilter, "clicked", accel_group,
                              GDK_F2, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);

  alignment16 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment16);
  gtk_container_add (GTK_CONTAINER (buttonAudioFilter), alignment16);

  hbox18 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox18);
  gtk_container_add (GTK_CONTAINER (alignment16), hbox18);

  label22 = gtk_label_new_with_mnemonic (QT_TR_NOOP("Filters"));
  gtk_widget_show (label22);
  gtk_box_pack_start (GTK_BOX (hbox18), label22, FALSE, FALSE, 0);

  hbox19 = gtk_hbox_new (FALSE, 6);
  gtk_widget_show (hbox19);
  gtk_box_pack_start (GTK_BOX (vbox8), hbox19, TRUE, TRUE, 0);

  CheckButtonTimeshift = gtk_check_button_new_with_mnemonic (QT_TR_NOOP("Shift:"));
  gtk_widget_show (CheckButtonTimeshift);
  gtk_box_pack_start (GTK_BOX (hbox19), CheckButtonTimeshift, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, CheckButtonTimeshift, QT_TR_NOOP("Enable time shift"), NULL);

  spinbuttonTimeShift_adj = gtk_adjustment_new (0, -99999, 99999, 1, 10, 0);
  spinbuttonTimeShift = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonTimeShift_adj), 1, 0);
  gtk_widget_show (spinbuttonTimeShift);
  gtk_box_pack_start (GTK_BOX (hbox19), spinbuttonTimeShift, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, spinbuttonTimeShift, QT_TR_NOOP("Audio/video time shift (ms)"), NULL);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonTimeShift), TRUE);

  label23 = gtk_label_new (QT_TR_NOOP("ms"));
  gtk_widget_show (label23);
  gtk_box_pack_start (GTK_BOX (hbox19), label23, FALSE, FALSE, 0);

  vbox13 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox13);
  gtk_box_pack_start (GTK_BOX (vbox9), vbox13, FALSE, FALSE, 0);

  label20 = gtk_label_new (QT_TR_NOOP("<b>Format</b>"));
  gtk_widget_show (label20);
  gtk_box_pack_start (GTK_BOX (vbox13), label20, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label20), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label20), 0, 0.5);

  alignment14 = gtk_alignment_new (0.5, 0.5, 1, 1);
  gtk_widget_show (alignment14);
  gtk_box_pack_start (GTK_BOX (vbox13), alignment14, TRUE, TRUE, 0);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment14), 3, 0, 12, 0);

  vbox15 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox15);
  gtk_container_add (GTK_CONTAINER (alignment14), vbox15);

  comboboxFormat = gtk_combo_box_new_text ();
  gtk_widget_show (comboboxFormat);
  gtk_box_pack_start (GTK_BOX (vbox15), comboboxFormat, TRUE, TRUE, 0);

  buttonConfM = gtk_button_new_with_mnemonic (QT_TR_NOOP("Configure"));
  gtk_widget_show (buttonConfM);
  gtk_box_pack_start (GTK_BOX (vbox15), buttonConfM, FALSE, FALSE, 0);

  guiDrawing = gtk_drawing_area_new ();
  gtk_widget_show (guiDrawing);
  gtk_box_pack_start (GTK_BOX (hbox14), guiDrawing, TRUE, TRUE, 0);
  gtk_widget_set_size_request (guiDrawing, 496, 288);
  gtk_widget_set_sensitive (guiDrawing, FALSE);

  table2 = gtk_table_new (3, 2, FALSE);
  gtk_widget_show (table2);
  gtk_box_pack_start (GTK_BOX (vbox10), table2, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (table2), 3);
  gtk_table_set_col_spacings (GTK_TABLE (table2), 6);

  hbox16 = gtk_hbox_new (FALSE, 6);
  gtk_widget_show (hbox16);
  gtk_table_attach (GTK_TABLE (table2), hbox16, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  label1 = gtk_label_new (QT_TR_NOOP("Frame: "));
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox16), label1, FALSE, FALSE, 0);

  boxCurFrame = gtk_entry_new ();
  gtk_widget_show (boxCurFrame);
  gtk_box_pack_start (GTK_BOX (hbox16), boxCurFrame, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, boxCurFrame, QT_TR_NOOP("Current frame"), NULL);
  gtk_entry_set_max_length (GTK_ENTRY (boxCurFrame), 8);
  gtk_entry_set_text (GTK_ENTRY (boxCurFrame), QT_TR_NOOP("0"));
  gtk_entry_set_width_chars (GTK_ENTRY (boxCurFrame), 8);

  labelTotalFrame = gtk_label_new (QT_TR_NOOP("/ 0000000"));
  gtk_widget_show (labelTotalFrame);
  gtk_box_pack_start (GTK_BOX (hbox16), labelTotalFrame, FALSE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS (labelTotalFrame, GTK_CAN_FOCUS);
  gtk_label_set_selectable (GTK_LABEL (labelTotalFrame), TRUE);
  gtk_misc_set_padding (GTK_MISC (labelTotalFrame), 5, 0);

  label4 = gtk_label_new (QT_TR_NOOP("Time: "));
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox16), label4, FALSE, FALSE, 0);

  boxCurTime = gtk_entry_new ();
  gtk_widget_show (boxCurTime);
  gtk_box_pack_start (GTK_BOX (hbox16), boxCurTime, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, boxCurTime, QT_TR_NOOP("Current time"), NULL);
  gtk_entry_set_text (GTK_ENTRY (boxCurTime), QT_TR_NOOP("00:00:00.000"));
  gtk_entry_set_width_chars (GTK_ENTRY (boxCurTime), 13);

  labelTotalTime = gtk_label_new (QT_TR_NOOP("/ 00:00:00,000"));
  gtk_widget_show (labelTotalTime);
  gtk_box_pack_start (GTK_BOX (hbox16), labelTotalTime, FALSE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS (labelTotalTime, GTK_CAN_FOCUS);
  gtk_label_set_selectable (GTK_LABEL (labelTotalTime), TRUE);
  gtk_misc_set_padding (GTK_MISC (labelTotalTime), 5, 0);

  labelFrameType = gtk_label_new (QT_TR_NOOP("Frame: ?"));
  gtk_widget_show (labelFrameType);
  gtk_box_pack_start (GTK_BOX (hbox16), labelFrameType, FALSE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS (labelFrameType, GTK_CAN_FOCUS);
  gtk_label_set_selectable (GTK_LABEL (labelFrameType), TRUE);

  sliderNavigate = gtk_markscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, 99.99, 0.01, 1, 0)));
  gtk_widget_show (sliderNavigate);gtk_scale_set_draw_value (GTK_SCALE (sliderNavigate), FALSE);
  gtk_table_attach (GTK_TABLE (table2), sliderNavigate, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  GTK_WIDGET_UNSET_FLAGS (sliderNavigate, GTK_CAN_FOCUS);
  gtk_scale_set_value_pos (GTK_SCALE (sliderNavigate), GTK_POS_LEFT);

  hbox10 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox10);
  gtk_table_attach (GTK_TABLE (table2), hbox10, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  buttonPlay = gtk_button_new ();
  gtk_widget_show (buttonPlay);
  gtk_box_pack_start (GTK_BOX (hbox10), buttonPlay, FALSE, FALSE, 0);
  GTK_WIDGET_UNSET_FLAGS (buttonPlay, GTK_CAN_FOCUS);
  gtk_tooltips_set_tip (tooltips, buttonPlay, QT_TR_NOOP("Play"), NULL);
  gtk_widget_add_accelerator (buttonPlay, "clicked", accel_group,
                              GDK_space, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);
  gtk_button_set_relief (GTK_BUTTON (buttonPlay), GTK_RELIEF_NONE);
  gtk_button_set_focus_on_click (GTK_BUTTON (buttonPlay), FALSE);

  image591 = create_pixmap (mainWindow, "play.xpm");
  gtk_widget_show (image591);
  gtk_container_add (GTK_CONTAINER (buttonPlay), image591);

  buttonStop = gtk_button_new ();
  gtk_widget_show (buttonStop);
  gtk_box_pack_start (GTK_BOX (hbox10), buttonStop, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonStop, QT_TR_NOOP("Stop"), NULL);
  gtk_button_set_relief (GTK_BUTTON (buttonStop), GTK_RELIEF_NONE);
  gtk_button_set_focus_on_click (GTK_BUTTON (buttonStop), FALSE);

  image5056 = create_pixmap (mainWindow, "stop.xpm");
  gtk_widget_show (image5056);
  gtk_container_add (GTK_CONTAINER (buttonStop), image5056);

  buttonPrevFrame = gtk_button_new ();
  gtk_widget_show (buttonPrevFrame);
  gtk_box_pack_start (GTK_BOX (hbox10), buttonPrevFrame, FALSE, FALSE, 0);
  GTK_WIDGET_UNSET_FLAGS (buttonPrevFrame, GTK_CAN_FOCUS);
  gtk_tooltips_set_tip (tooltips, buttonPrevFrame, QT_TR_NOOP("Previous frame"), NULL);
  gtk_widget_add_accelerator (buttonPrevFrame, "clicked", accel_group,
                              GDK_KP_4, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);
  gtk_button_set_relief (GTK_BUTTON (buttonPrevFrame), GTK_RELIEF_NONE);
  gtk_button_set_focus_on_click (GTK_BUTTON (buttonPrevFrame), FALSE);

  image593 = create_pixmap (mainWindow, "backward.xpm");
  gtk_widget_show (image593);
  gtk_container_add (GTK_CONTAINER (buttonPrevFrame), image593);

  buttonNextFrame = gtk_button_new ();
  gtk_widget_show (buttonNextFrame);
  gtk_box_pack_start (GTK_BOX (hbox10), buttonNextFrame, FALSE, FALSE, 0);
  GTK_WIDGET_UNSET_FLAGS (buttonNextFrame, GTK_CAN_FOCUS);
  gtk_tooltips_set_tip (tooltips, buttonNextFrame, QT_TR_NOOP("Next frame"), NULL);
  gtk_widget_add_accelerator (buttonNextFrame, "clicked", accel_group,
                              GDK_KP_6, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);
  gtk_button_set_relief (GTK_BUTTON (buttonNextFrame), GTK_RELIEF_NONE);
  gtk_button_set_focus_on_click (GTK_BUTTON (buttonNextFrame), FALSE);

  image594 = create_pixmap (mainWindow, "forward.xpm");
  gtk_widget_show (image594);
  gtk_container_add (GTK_CONTAINER (buttonNextFrame), image594);

  buttonPrevKFrame = gtk_button_new ();
  gtk_widget_show (buttonPrevKFrame);
  gtk_box_pack_start (GTK_BOX (hbox10), buttonPrevKFrame, FALSE, FALSE, 0);
  GTK_WIDGET_UNSET_FLAGS (buttonPrevKFrame, GTK_CAN_FOCUS);
  gtk_tooltips_set_tip (tooltips, buttonPrevKFrame, QT_TR_NOOP("Previous keyframe"), NULL);
  gtk_widget_add_accelerator (buttonPrevKFrame, "clicked", accel_group,
                              GDK_KP_2, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);
  gtk_button_set_relief (GTK_BUTTON (buttonPrevKFrame), GTK_RELIEF_NONE);
  gtk_button_set_focus_on_click (GTK_BUTTON (buttonPrevKFrame), FALSE);

  image595 = create_pixmap (mainWindow, "Kbackward.xpm");
  gtk_widget_show (image595);
  gtk_container_add (GTK_CONTAINER (buttonPrevKFrame), image595);

  buttonNextKFrame = gtk_button_new ();
  gtk_widget_show (buttonNextKFrame);
  gtk_box_pack_start (GTK_BOX (hbox10), buttonNextKFrame, FALSE, FALSE, 0);
  GTK_WIDGET_UNSET_FLAGS (buttonNextKFrame, GTK_CAN_FOCUS);
  gtk_tooltips_set_tip (tooltips, buttonNextKFrame, QT_TR_NOOP("Next keyframe"), NULL);
  gtk_widget_add_accelerator (buttonNextKFrame, "clicked", accel_group,
                              GDK_KP_8, (GdkModifierType)( 0 ),
                              GTK_ACCEL_VISIBLE);
  gtk_button_set_relief (GTK_BUTTON (buttonNextKFrame), GTK_RELIEF_NONE);
  gtk_button_set_focus_on_click (GTK_BUTTON (buttonNextKFrame), FALSE);

  image596 = create_pixmap (mainWindow, "Kforward.xpm");
  gtk_widget_show (image596);
  gtk_container_add (GTK_CONTAINER (buttonNextKFrame), image596);

  buttonMarkA = gtk_button_new ();
  gtk_widget_show (buttonMarkA);
  gtk_box_pack_start (GTK_BOX (hbox10), buttonMarkA, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonMarkA, QT_TR_NOOP("Selection: start"), NULL);
  gtk_button_set_relief (GTK_BUTTON (buttonMarkA), GTK_RELIEF_NONE);
  gtk_button_set_focus_on_click (GTK_BUTTON (buttonMarkA), FALSE);

  image2306 = create_pixmap (mainWindow, "markA.xpm");
  gtk_widget_show (image2306);
  gtk_container_add (GTK_CONTAINER (buttonMarkA), image2306);

  buttonMarkB = gtk_button_new ();
  gtk_widget_show (buttonMarkB);
  gtk_box_pack_start (GTK_BOX (hbox10), buttonMarkB, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonMarkB, QT_TR_NOOP("Selection: end"), NULL);
  gtk_button_set_relief (GTK_BUTTON (buttonMarkB), GTK_RELIEF_NONE);
  gtk_button_set_focus_on_click (GTK_BUTTON (buttonMarkB), FALSE);

  image2307 = create_pixmap (mainWindow, "markB.xpm");
  gtk_widget_show (image2307);
  gtk_container_add (GTK_CONTAINER (buttonMarkB), image2307);

  buttonPrevBlack = gtk_button_new ();
  gtk_widget_show (buttonPrevBlack);
  gtk_box_pack_start (GTK_BOX (hbox10), buttonPrevBlack, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonPrevBlack, QT_TR_NOOP("Previous black frame"), NULL);
  gtk_button_set_relief (GTK_BUTTON (buttonPrevBlack), GTK_RELIEF_NONE);
  gtk_button_set_focus_on_click (GTK_BUTTON (buttonPrevBlack), FALSE);

  image2308 = create_pixmap (mainWindow, "xpm_prevblack.xpm");
  gtk_widget_show (image2308);
  gtk_container_add (GTK_CONTAINER (buttonPrevBlack), image2308);

  buttonNextBlack = gtk_button_new ();
  gtk_widget_show (buttonNextBlack);
  gtk_box_pack_start (GTK_BOX (hbox10), buttonNextBlack, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonNextBlack, QT_TR_NOOP("Next black frame"), NULL);
  gtk_button_set_relief (GTK_BUTTON (buttonNextBlack), GTK_RELIEF_NONE);
  gtk_button_set_focus_on_click (GTK_BUTTON (buttonNextBlack), FALSE);

  image2309 = create_pixmap (mainWindow, "xpm_nextblack.xpm");
  gtk_widget_show (image2309);
  gtk_container_add (GTK_CONTAINER (buttonNextBlack), image2309);

  buttonBegin = gtk_button_new ();
  gtk_widget_show (buttonBegin);
  gtk_box_pack_start (GTK_BOX (hbox10), buttonBegin, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonBegin, QT_TR_NOOP("First frame"), NULL);
  gtk_button_set_relief (GTK_BUTTON (buttonBegin), GTK_RELIEF_NONE);
  gtk_button_set_focus_on_click (GTK_BUTTON (buttonBegin), FALSE);

  image5057 = create_pixmap (mainWindow, "begin.xpm");
  gtk_widget_show (image5057);
  gtk_container_add (GTK_CONTAINER (buttonBegin), image5057);

  buttonEnd = gtk_button_new ();
  gtk_widget_show (buttonEnd);
  gtk_box_pack_start (GTK_BOX (hbox10), buttonEnd, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonEnd, QT_TR_NOOP("Last frame"), NULL);
  gtk_button_set_relief (GTK_BUTTON (buttonEnd), GTK_RELIEF_NONE);
  gtk_button_set_focus_on_click (GTK_BUTTON (buttonEnd), FALSE);

  image2245 = create_pixmap (mainWindow, "end.xpm");
  gtk_widget_show (image2245);
  gtk_container_add (GTK_CONTAINER (buttonEnd), image2245);

  vbox14 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox14);
  gtk_table_attach (GTK_TABLE (table2), vbox14, 1, 2, 1, 3,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label21 = gtk_label_new (QT_TR_NOOP("<b>Selection</b>"));
  gtk_widget_show (label21);
  gtk_box_pack_start (GTK_BOX (vbox14), label21, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label21), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label21), 0, 0.5);

  alignment15 = gtk_alignment_new (0.5, 0.5, 1, 1);
  gtk_widget_show (alignment15);
  gtk_box_pack_start (GTK_BOX (vbox14), alignment15, TRUE, TRUE, 0);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment15), 3, 0, 12, 0);

  table1 = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (alignment15), table1);
  gtk_table_set_row_spacings (GTK_TABLE (table1), 3);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 3);

  buttonGotoB = gtk_button_new_with_mnemonic (QT_TR_NOOP("B:"));
  gtk_widget_show (buttonGotoB);
  gtk_table_attach (GTK_TABLE (table1), buttonGotoB, 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_size_request (buttonGotoB, -1, 20);
  GTK_WIDGET_UNSET_FLAGS (buttonGotoB, GTK_CAN_FOCUS);
  gtk_tooltips_set_tip (tooltips, buttonGotoB, QT_TR_NOOP("Go to marker B"), NULL);

  buttonGotoA = gtk_button_new_with_mnemonic (QT_TR_NOOP("A:"));
  gtk_widget_show (buttonGotoA);
  gtk_table_attach (GTK_TABLE (table1), buttonGotoA, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_size_request (buttonGotoA, -1, 20);
  GTK_WIDGET_UNSET_FLAGS (buttonGotoA, GTK_CAN_FOCUS);
  gtk_tooltips_set_tip (tooltips, buttonGotoA, QT_TR_NOOP("Go to marker A"), NULL);

  labelMarkB = gtk_label_new (QT_TR_NOOP(" 0000000"));
  gtk_widget_show (labelMarkB);
  gtk_table_attach (GTK_TABLE (table1), labelMarkB, 1, 2, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  GTK_WIDGET_SET_FLAGS (labelMarkB, GTK_CAN_FOCUS);
  gtk_label_set_selectable (GTK_LABEL (labelMarkB), TRUE);

  labelMarkA = gtk_label_new (QT_TR_NOOP(" 0000000"));
  gtk_widget_show (labelMarkA);
  gtk_table_attach (GTK_TABLE (table1), labelMarkA, 1, 2, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  GTK_WIDGET_SET_FLAGS (labelMarkA, GTK_CAN_FOCUS);
  gtk_label_set_selectable (GTK_LABEL (labelMarkA), TRUE);

  jogg = jog_shuttle_new ();
  gtk_widget_set_size_request (jogg, -1, 16); 
  gtk_widget_show (jogg);
  gtk_table_attach (GTK_TABLE (table2), jogg, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  GTK_WIDGET_UNSET_FLAGS (jogg, GTK_CAN_FOCUS);
  GTK_WIDGET_UNSET_FLAGS (jogg, GTK_CAN_DEFAULT);


  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (mainWindow, mainWindow, "mainWindow");
  GLADE_HOOKUP_OBJECT (mainWindow, vbox1, "vbox1");
  GLADE_HOOKUP_OBJECT (mainWindow, menuBar, "menuBar");
  GLADE_HOOKUP_OBJECT (mainWindow, menuitem1, "menuitem1");
  GLADE_HOOKUP_OBJECT (mainWindow, menuitem1_menu, "menuitem1_menu");
  GLADE_HOOKUP_OBJECT (mainWindow, open_video1, "open_video1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8523, "image8523");
  GLADE_HOOKUP_OBJECT (mainWindow, append_video1, "append_video1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8524, "image8524");
  GLADE_HOOKUP_OBJECT (mainWindow, save_stuff, "save_stuff");
  GLADE_HOOKUP_OBJECT (mainWindow, image8525, "image8525");
  GLADE_HOOKUP_OBJECT (mainWindow, save_stuff_menu, "save_stuff_menu");
  GLADE_HOOKUP_OBJECT (mainWindow, save_as_avi1, "save_as_avi1");
  GLADE_HOOKUP_OBJECT (mainWindow, separator1, "separator1");
  GLADE_HOOKUP_OBJECT (mainWindow, save_image1, "save_image1");
  GLADE_HOOKUP_OBJECT (mainWindow, save_jpg_image1, "save_jpg_image1");
  GLADE_HOOKUP_OBJECT (mainWindow, save_selection_as_jpegs1, "save_selection_as_jpegs1");
  GLADE_HOOKUP_OBJECT (mainWindow, close1, "close1");
  GLADE_HOOKUP_OBJECT (mainWindow, separator3, "separator3");
  GLADE_HOOKUP_OBJECT (mainWindow, save_project1, "save_project1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8526, "image8526");
  GLADE_HOOKUP_OBJECT (mainWindow, save_project_as1, "save_project_as1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8527, "image8527");
  GLADE_HOOKUP_OBJECT (mainWindow, run_script1, "run_script1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8528, "image8528");
  GLADE_HOOKUP_OBJECT (mainWindow, add_to_joblist1, "add_to_joblist1");
  GLADE_HOOKUP_OBJECT (mainWindow, joblist1, "joblist1");
  GLADE_HOOKUP_OBJECT (mainWindow, separator16, "separator16");
  GLADE_HOOKUP_OBJECT (mainWindow, connect_to_avsproxy1, "connect_to_avsproxy1");
  GLADE_HOOKUP_OBJECT (mainWindow, separator18, "separator18");
  GLADE_HOOKUP_OBJECT (mainWindow, video_informations1, "video_informations1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8529, "image8529");
  GLADE_HOOKUP_OBJECT (mainWindow, separator4, "separator4");
  GLADE_HOOKUP_OBJECT (mainWindow, quit1, "quit1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8530, "image8530");
  GLADE_HOOKUP_OBJECT (mainWindow, menuitem2, "menuitem2");
  GLADE_HOOKUP_OBJECT (mainWindow, menuitem2_menu, "menuitem2_menu");
  GLADE_HOOKUP_OBJECT (mainWindow, reset_edits1, "reset_edits1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8531, "image8531");
  GLADE_HOOKUP_OBJECT (mainWindow, separator5, "separator5");
  GLADE_HOOKUP_OBJECT (mainWindow, cut1, "cut1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8532, "image8532");
  GLADE_HOOKUP_OBJECT (mainWindow, copy1, "copy1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8533, "image8533");
  GLADE_HOOKUP_OBJECT (mainWindow, paste1, "paste1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8534, "image8534");
  GLADE_HOOKUP_OBJECT (mainWindow, delete1, "delete1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8535, "image8535");
  GLADE_HOOKUP_OBJECT (mainWindow, separator6, "separator6");
  GLADE_HOOKUP_OBJECT (mainWindow, set_marker_a1, "set_marker_a1");
  GLADE_HOOKUP_OBJECT (mainWindow, set_marker_b1, "set_marker_b1");
  GLADE_HOOKUP_OBJECT (mainWindow, separator7, "separator7");
  GLADE_HOOKUP_OBJECT (mainWindow, preferences1, "preferences1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8536, "image8536");
  GLADE_HOOKUP_OBJECT (mainWindow, view1, "view1");
  GLADE_HOOKUP_OBJECT (mainWindow, view1_menu, "view1_menu");
  GLADE_HOOKUP_OBJECT (mainWindow, toolbar1, "toolbar1");
  GLADE_HOOKUP_OBJECT (mainWindow, sidebar1, "sidebar1");
  GLADE_HOOKUP_OBJECT (mainWindow, separator8, "separator8");
  GLADE_HOOKUP_OBJECT (mainWindow, zoom_1_4, "zoom_1_4");
  GLADE_HOOKUP_OBJECT (mainWindow, zoom_1_2, "zoom_1_2");
  GLADE_HOOKUP_OBJECT (mainWindow, zoom_1_1, "zoom_1_1");
  GLADE_HOOKUP_OBJECT (mainWindow, zoom_2_1, "zoom_2_1");
  GLADE_HOOKUP_OBJECT (mainWindow, video_1, "video_1");
  GLADE_HOOKUP_OBJECT (mainWindow, video_1_menu, "video_1_menu");
  GLADE_HOOKUP_OBJECT (mainWindow, decoder_options1, "decoder_options1");
  GLADE_HOOKUP_OBJECT (mainWindow, set_postprocessing1, "set_postprocessing1");
  GLADE_HOOKUP_OBJECT (mainWindow, separator9, "separator9");
  GLADE_HOOKUP_OBJECT (mainWindow, preview1, "preview1");
  GLADE_HOOKUP_OBJECT (mainWindow, display_output1, "display_output1");
  GLADE_HOOKUP_OBJECT (mainWindow, separator10, "separator10");
  GLADE_HOOKUP_OBJECT (mainWindow, change_fps, "change_fps");
  GLADE_HOOKUP_OBJECT (mainWindow, videoencoder, "videoencoder");
  GLADE_HOOKUP_OBJECT (mainWindow, filters1, "filters1");
  GLADE_HOOKUP_OBJECT (mainWindow, audio1, "audio1");
  GLADE_HOOKUP_OBJECT (mainWindow, audio1_menu, "audio1_menu");
  GLADE_HOOKUP_OBJECT (mainWindow, main_audio, "main_audio");
  GLADE_HOOKUP_OBJECT (mainWindow, second_audio_track1, "second_audio_track1");
  GLADE_HOOKUP_OBJECT (mainWindow, build_vbr_time_map1, "build_vbr_time_map1");
  GLADE_HOOKUP_OBJECT (mainWindow, separator11, "separator11");
  GLADE_HOOKUP_OBJECT (mainWindow, save_audio1, "save_audio1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8537, "image8537");
  GLADE_HOOKUP_OBJECT (mainWindow, audio_encoder1, "audio_encoder1");
  GLADE_HOOKUP_OBJECT (mainWindow, filters2, "filters2");
  GLADE_HOOKUP_OBJECT (mainWindow, tools1, "tools1");
  GLADE_HOOKUP_OBJECT (mainWindow, tools1_menu, "tools1_menu");
  GLADE_HOOKUP_OBJECT (mainWindow, calculator1, "calculator1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8538, "image8538");
  GLADE_HOOKUP_OBJECT (mainWindow, separator12, "separator12");
  GLADE_HOOKUP_OBJECT (mainWindow, rebuild_frames, "rebuild_frames");
  GLADE_HOOKUP_OBJECT (mainWindow, check_frames, "check_frames");
  GLADE_HOOKUP_OBJECT (mainWindow, bitrate_histogram1, "bitrate_histogram1");
  GLADE_HOOKUP_OBJECT (mainWindow, item1, "item1");
  GLADE_HOOKUP_OBJECT (mainWindow, separator13, "separator13");
  GLADE_HOOKUP_OBJECT (mainWindow, v2v, "v2v");
  GLADE_HOOKUP_OBJECT (mainWindow, ocr_vobsub_2_srt, "ocr_vobsub_2_srt");
  GLADE_HOOKUP_OBJECT (mainWindow, ocr_dvb, "ocr_dvb");
  GLADE_HOOKUP_OBJECT (mainWindow, edit_glyph1, "edit_glyph1");
  GLADE_HOOKUP_OBJECT (mainWindow, separator17, "separator17");
  GLADE_HOOKUP_OBJECT (mainWindow, see_hex1, "see_hex1");
  GLADE_HOOKUP_OBJECT (mainWindow, help1, "help1");
  GLADE_HOOKUP_OBJECT (mainWindow, go1, "go1");
  GLADE_HOOKUP_OBJECT (mainWindow, go1_menu, "go1_menu");
  GLADE_HOOKUP_OBJECT (mainWindow, play_video1, "play_video1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8539, "image8539");
  GLADE_HOOKUP_OBJECT (mainWindow, previous_frame1, "previous_frame1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8540, "image8540");
  GLADE_HOOKUP_OBJECT (mainWindow, next_frame1, "next_frame1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8541, "image8541");
  GLADE_HOOKUP_OBJECT (mainWindow, previous_intra_frame1, "previous_intra_frame1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8542, "image8542");
  GLADE_HOOKUP_OBJECT (mainWindow, next_intra_frame1, "next_intra_frame1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8543, "image8543");
  GLADE_HOOKUP_OBJECT (mainWindow, search_previous_black_frame1, "search_previous_black_frame1");
  GLADE_HOOKUP_OBJECT (mainWindow, search_next_black_frame1, "search_next_black_frame1");
  GLADE_HOOKUP_OBJECT (mainWindow, first_frame1, "first_frame1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8544, "image8544");
  GLADE_HOOKUP_OBJECT (mainWindow, last_frame1, "last_frame1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8545, "image8545");
  GLADE_HOOKUP_OBJECT (mainWindow, separator14, "separator14");
  GLADE_HOOKUP_OBJECT (mainWindow, go_to_marker_a1, "go_to_marker_a1");
  GLADE_HOOKUP_OBJECT (mainWindow, go_to_marker_b1, "go_to_marker_b1");
  GLADE_HOOKUP_OBJECT (mainWindow, separator15, "separator15");
  GLADE_HOOKUP_OBJECT (mainWindow, jum_to_frame1, "jum_to_frame1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8546, "image8546");
  GLADE_HOOKUP_OBJECT (mainWindow, jump_to_time1, "jump_to_time1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8547, "image8547");
  GLADE_HOOKUP_OBJECT (mainWindow, custom1, "custom1");
  GLADE_HOOKUP_OBJECT (mainWindow, help2, "help2");
  GLADE_HOOKUP_OBJECT (mainWindow, help2_menu, "help2_menu");
  GLADE_HOOKUP_OBJECT (mainWindow, plugins1, "plugins1");
  GLADE_HOOKUP_OBJECT (mainWindow, show_builtin_support1, "show_builtin_support1");
  GLADE_HOOKUP_OBJECT (mainWindow, about1, "about1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8548, "image8548");
  GLADE_HOOKUP_OBJECT (mainWindow, toolbar2, "toolbar2");
  GLADE_HOOKUP_OBJECT (mainWindow, menutoolbuttonOpen, "menutoolbuttonOpen");
  GLADE_HOOKUP_OBJECT (mainWindow, toolitem11, "toolitem11");
  GLADE_HOOKUP_OBJECT (mainWindow, toolbuttonSave, "toolbuttonSave");
  GLADE_HOOKUP_OBJECT (mainWindow, toolbuttonInfo, "toolbuttonInfo");
  GLADE_HOOKUP_OBJECT (mainWindow, separatortoolitem1, "separatortoolitem1");
  GLADE_HOOKUP_OBJECT (mainWindow, toolbuttonCalc, "toolbuttonCalc");
  GLADE_HOOKUP_OBJECT (mainWindow, separatortoolitem2, "separatortoolitem2");
  GLADE_HOOKUP_OBJECT (mainWindow, toolitem12, "toolitem12");
  GLADE_HOOKUP_OBJECT (mainWindow, comboboxPreview, "comboboxPreview");
  GLADE_HOOKUP_OBJECT (mainWindow, toolitem13, "toolitem13");
  GLADE_HOOKUP_OBJECT (mainWindow, togglebutton1, "togglebutton1");
  GLADE_HOOKUP_OBJECT (mainWindow, image8272, "image8272");
  GLADE_HOOKUP_OBJECT (mainWindow, toolitem14, "toolitem14");
  GLADE_HOOKUP_OBJECT (mainWindow, hscalVolume, "hscalVolume");
  GLADE_HOOKUP_OBJECT (mainWindow, vbox10, "vbox10");
  GLADE_HOOKUP_OBJECT (mainWindow, hbox14, "hbox14");
  GLADE_HOOKUP_OBJECT (mainWindow, alignment19, "alignment19");
  GLADE_HOOKUP_OBJECT (mainWindow, vbox9, "vbox9");
  GLADE_HOOKUP_OBJECT (mainWindow, vbox11, "vbox11");
  GLADE_HOOKUP_OBJECT (mainWindow, label18, "label18");
  GLADE_HOOKUP_OBJECT (mainWindow, alignment12, "alignment12");
  GLADE_HOOKUP_OBJECT (mainWindow, vbox7, "vbox7");
  GLADE_HOOKUP_OBJECT (mainWindow, comboboxVideo, "comboboxVideo");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonConfV, "buttonConfV");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonFilters, "buttonFilters");
  GLADE_HOOKUP_OBJECT (mainWindow, vbox12, "vbox12");
  GLADE_HOOKUP_OBJECT (mainWindow, label19, "label19");
  GLADE_HOOKUP_OBJECT (mainWindow, alignment13, "alignment13");
  GLADE_HOOKUP_OBJECT (mainWindow, vbox8, "vbox8");
  GLADE_HOOKUP_OBJECT (mainWindow, comboboxAudio, "comboboxAudio");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonConfA, "buttonConfA");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonAudioFilter, "buttonAudioFilter");
  GLADE_HOOKUP_OBJECT (mainWindow, alignment16, "alignment16");
  GLADE_HOOKUP_OBJECT (mainWindow, hbox18, "hbox18");
  GLADE_HOOKUP_OBJECT (mainWindow, label22, "label22");
  GLADE_HOOKUP_OBJECT (mainWindow, hbox19, "hbox19");
  GLADE_HOOKUP_OBJECT (mainWindow, CheckButtonTimeshift, "CheckButtonTimeshift");
  GLADE_HOOKUP_OBJECT (mainWindow, spinbuttonTimeShift, "spinbuttonTimeShift");
  GLADE_HOOKUP_OBJECT (mainWindow, label23, "label23");
  GLADE_HOOKUP_OBJECT (mainWindow, vbox13, "vbox13");
  GLADE_HOOKUP_OBJECT (mainWindow, label20, "label20");
  GLADE_HOOKUP_OBJECT (mainWindow, alignment14, "alignment14");
  GLADE_HOOKUP_OBJECT (mainWindow, vbox15, "vbox15");
  GLADE_HOOKUP_OBJECT (mainWindow, comboboxFormat, "comboboxFormat");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonConfM, "buttonConfM");
  GLADE_HOOKUP_OBJECT (mainWindow, guiDrawing, "guiDrawing");
  GLADE_HOOKUP_OBJECT (mainWindow, table2, "table2");
  GLADE_HOOKUP_OBJECT (mainWindow, hbox16, "hbox16");
  GLADE_HOOKUP_OBJECT (mainWindow, label1, "label1");
  GLADE_HOOKUP_OBJECT (mainWindow, boxCurFrame, "boxCurFrame");
  GLADE_HOOKUP_OBJECT (mainWindow, labelTotalFrame, "labelTotalFrame");
  GLADE_HOOKUP_OBJECT (mainWindow, label4, "label4");
  GLADE_HOOKUP_OBJECT (mainWindow, boxCurTime, "boxCurTime");
  GLADE_HOOKUP_OBJECT (mainWindow, labelTotalTime, "labelTotalTime");
  GLADE_HOOKUP_OBJECT (mainWindow, labelFrameType, "labelFrameType");
  GLADE_HOOKUP_OBJECT (mainWindow, sliderNavigate, "sliderNavigate");
  GLADE_HOOKUP_OBJECT (mainWindow, hbox10, "hbox10");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonPlay, "buttonPlay");
  GLADE_HOOKUP_OBJECT (mainWindow, image591, "image591");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonStop, "buttonStop");
  GLADE_HOOKUP_OBJECT (mainWindow, image5056, "image5056");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonPrevFrame, "buttonPrevFrame");
  GLADE_HOOKUP_OBJECT (mainWindow, image593, "image593");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonNextFrame, "buttonNextFrame");
  GLADE_HOOKUP_OBJECT (mainWindow, image594, "image594");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonPrevKFrame, "buttonPrevKFrame");
  GLADE_HOOKUP_OBJECT (mainWindow, image595, "image595");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonNextKFrame, "buttonNextKFrame");
  GLADE_HOOKUP_OBJECT (mainWindow, image596, "image596");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonMarkA, "buttonMarkA");
  GLADE_HOOKUP_OBJECT (mainWindow, image2306, "image2306");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonMarkB, "buttonMarkB");
  GLADE_HOOKUP_OBJECT (mainWindow, image2307, "image2307");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonPrevBlack, "buttonPrevBlack");
  GLADE_HOOKUP_OBJECT (mainWindow, image2308, "image2308");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonNextBlack, "buttonNextBlack");
  GLADE_HOOKUP_OBJECT (mainWindow, image2309, "image2309");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonBegin, "buttonBegin");
  GLADE_HOOKUP_OBJECT (mainWindow, image5057, "image5057");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonEnd, "buttonEnd");
  GLADE_HOOKUP_OBJECT (mainWindow, image2245, "image2245");
  GLADE_HOOKUP_OBJECT (mainWindow, vbox14, "vbox14");
  GLADE_HOOKUP_OBJECT (mainWindow, label21, "label21");
  GLADE_HOOKUP_OBJECT (mainWindow, alignment15, "alignment15");
  GLADE_HOOKUP_OBJECT (mainWindow, table1, "table1");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonGotoB, "buttonGotoB");
  GLADE_HOOKUP_OBJECT (mainWindow, buttonGotoA, "buttonGotoA");
  GLADE_HOOKUP_OBJECT (mainWindow, labelMarkB, "labelMarkB");
  GLADE_HOOKUP_OBJECT (mainWindow, labelMarkA, "labelMarkA");
  GLADE_HOOKUP_OBJECT (mainWindow, jogg, "jogg");
  GLADE_HOOKUP_OBJECT_NO_REF (mainWindow, tooltips, "tooltips");

  gtk_window_add_accel_group (GTK_WINDOW (mainWindow), accel_group);

  return mainWindow;
}
