/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ADM_VID_PARTIAL_PARAM_H
#define ADM_VID_PARTIAL_PARAM_H

typedef struct  PARTIAL_CONFIG
{
  uint32_t _start,_end;
  uint32_t _confsize;
  uint32_t _tag;

}PARTIAL_CONFIG;

#endif
